#!/usr/bin/env python

# Inspire by Conbagui's tutorial found here: https://web.archive.org/web/20150317020228/http://gimpchat.com/viewtopic.php?f=23&t=11969

# author: Pocholo
# date: 10/8/2021

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder


# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>. 


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_denim_patch_text_effect(customtext, fontname, color):
	
	#Initiates the defaults
    pdb.gimp_context_set_defaults()
    
    #Create the denim background
    img = pdb.gimp_image_new(1600, 1200, RGB)
    jeanLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Jean", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, jeanLayer, 0)
    pdb.gimp_drawable_fill(jeanLayer, FILL_WHITE)
    pdb.plug_in_gmic_qt(img, jeanLayer, 1, 0, "Denim_samj 5,2,0,40,40,25,50,0,0,120,163,255") 
    
    #Process
    pdb.gimp_display_new(img)
    
    #Create a new layer fill with Noise and change the layer mode
    noiseLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Noise and stripes", 20, LAYER_MODE_DODGE)
    pdb.gimp_image_add_layer(img, noiseLayer, 0)
    pdb.gimp_context_set_foreground((0, 58, 192))
    pdb.gimp_drawable_fill(noiseLayer, FILL_FOREGROUND)
    pdb.plug_in_hsv_noise(img, noiseLayer, 1, 180, 0, 255)

    #Use g'mic stripe filter
    pdb.plug_in_gmic_qt(img, noiseLayer, 1, 0, "jeje_strip 45,100,100,2") 

    #Merge all layers
    jeanLayer = pdb.gimp_image_merge_visible_layers(img, CLIP_TO_BOTTOM_LAYER)
    jeanLayer = pdb.gimp_image_get_layer_by_name(img, "Jean")
    pdb.gimp_hue_saturation(jeanLayer, 0, 10, 21, 100)
    non_empty = pdb.gimp_edit_copy(jeanLayer)
    
    #Patch #1
    patch1Layer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Patch #1", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, patch1Layer, 0)
    pdb.gimp_drawable_fill(patch1Layer, FILL_TRANSPARENT)
    pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, 120, 165, 1350, 850)
    pdb.gimp_context_set_pattern("Rain")
    pdb.gimp_edit_bucket_fill(patch1Layer,  
                      BUCKET_FILL_PATTERN,  #fill_mode
                        LAYER_MODE_NORMAL,  #paint_mode
                                      100,  #opacity
                                      255,  #threshold
                                    FALSE,  #sample_merged
                                        0,  #x
                                        0)  #y
    pdb.gimp_selection_none(img)
    patch1Layer = pdb.gimp_item_transform_rotate(patch1Layer, -6.5, TRUE, 795, 590)
    pdb.gimp_layer_resize_to_image_size(patch1Layer)
    pdb.gimp_image_select_item(img, 2, patch1Layer)
    pdb.plug_in_sel2path(img, patch1Layer)
    pdb.gimp_selection_none(img)
    
    #Outer Glow
    outglowLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Outer Glow", 100, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, outglowLayer, 1)
    pdb.gimp_drawable_fill(outglowLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, 120, 165, 1350, 850)    
    pdb.gimp_selection_grow(img, 10)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_edit_bucket_fill(outglowLayer,  
                            BUCKET_FILL_FG,  #fill_mode
                         LAYER_MODE_NORMAL,  #paint_mode
                                       100,  #opacity
                                       255,  #threshold
                                     FALSE,  #sample_merged
                                         0,  #x
                                         0)  #y
    pdb.gimp_selection_none(img)
    outglowLayer = pdb.gimp_item_transform_rotate(outglowLayer, -6.5, TRUE, 795, 590)
    pdb.gimp_layer_resize_to_image_size(outglowLayer)    
    pdb.plug_in_gauss(img, outglowLayer, 30, 30, 1)   #method=1
                 
    #Create the fray    
    pdb.gimp_context_set_brush("Acrylic 01")    
    pdb.gimp_context_set_brush_size(140)
    pdb.gimp_context_set_brush_aspect_ratio(19)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.05)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_context_set_dynamics("Random Size/Angle")
    pdb.gimp_context_set_paint_method("gimp-eraser")
    pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
    vectors1 = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))   
    pdb.gimp_drawable_edit_stroke_item(patch1Layer, vectors1)
        
    #Patch #2
    patch2Layer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Patch #2", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, patch2Layer, 0)
    pdb.gimp_drawable_fill(patch2Layer, FILL_TRANSPARENT)
    pdb.gimp_image_select_item(img, 2, vectors1)
    pdb.gimp_selection_shrink(img, 30)
    pdb.plug_in_sel2path(img, patch2Layer)  
    pdb.gimp_context_set_foreground((66, 107, 215))
    pdb.gimp_edit_bucket_fill(patch2Layer,  
                           BUCKET_FILL_FG,  #fill_mode
                        LAYER_MODE_NORMAL,  #paint_mode
                                      100,  #opacity
                                      255,  #threshold
                                    FALSE,  #sample_merged
                                        0,  #x
                                        0)  #y
    
    pdb.plug_in_gmic_qt(img, patch2Layer, 1, 0, "texturize_canvas 256,20,1")   
    pdb.gimp_hue_saturation(patch2Layer, 0, -10, -10, 100)
     
    #Create the Patch stitches
    stitches1 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Patch stitches", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, stitches1, 0)
    pdb.gimp_drawable_fill(stitches1, FILL_TRANSPARENT)	
    pdb.gimp_selection_shrink(img, 20)
    pdb.plug_in_sel2path(img, stitches1)	
    pdb.gimp_selection_none(img)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_context_set_foreground((72, 116, 255))	
    pdb.gimp_context_set_line_cap_style(1)
    pdb.gimp_context_set_line_dash_offset(0)
    pdb.gimp_context_set_line_dash_pattern(3, [1.5, 3])
    pdb.gimp_context_set_line_join_style(JOIN_ROUND)
    pdb.gimp_context_set_line_miter_limit(10)
    pdb.gimp_context_set_line_width(8)
    pdb.gimp_context_set_line_width_unit(PIXELS)
    pdb.gimp_context_set_stroke_method (STROKE_LINE)
    vectors3 = pdb.gimp_image_get_vectors_by_name(img, (_("Selection")+' #2'))
    pdb.gimp_drawable_edit_stroke_item(stitches1, vectors3)
    
    #Bevel and Emboss	
    pdb.python_layerfx_bevel_emboss(img, stitches1, 
												 3,   #style - pillow emboss
												65,   #depth 
												 0,   #direction 
											     4,   #size 
												 2,   #soften
											    90,   #angle 
												30,   #altitude 
												 0,   #gloss_contour - linear
								         ("White"),   #highlight_color 
												 4,   #highlight_mode - screen
												75,   #highlight_opacity
									     ("Black"),   #shadow_color 
												 3,   #shadow_mode-multiply
												80,   #shadow_opacity 
												 0,   #surface_contour - linear
											 FALSE,   #use_texture 
									   "Dried mud",   #pattern 
											   100,   #scale 
											   100,   #tex_depth 
											  TRUE,   #invert 
											 FALSE)   #merge		
    
    #Create the text		
    size = 400
    textW = size/0.6
    textH = size/0.6
    pdb.gimp_context_set_foreground("White")
    textLayer = pdb.gimp_text_layer_new(img, customtext, fontname, size, PIXELS)
    pdb.gimp_image_add_layer(img, textLayer, 0)
    pdb.gimp_layer_add_alpha(textLayer)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    	
	#Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)
			
	#Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    pdb.gimp_layer_set_mode(textLayer, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(textLayer, 100)  
    #Rename the text layer
    pdb.gimp_item_set_name(textLayer, "Text")
    #Rotate text
    textLayer = pdb.gimp_item_transform_rotate(textLayer, -6.5, TRUE, 795, 590)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Create the effect on the text
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_context_set_pattern("Rain")  
    pdb.gimp_edit_bucket_fill(textLayer,  
                    BUCKET_FILL_PATTERN,  #fill_mode
                      LAYER_MODE_NORMAL,  #paint_mode
                                    100,  #opacity
                                    255,  #threshold
                                  FALSE,  #sample_merged
                                      0,  #x
                                      0)  #y
    
    fabLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Fabric", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, fabLayer, 0)
    pdb.gimp_drawable_fill(fabLayer, FILL_TRANSPARENT)	  
    pdb.gimp_selection_shrink(img, 10)
    pdb.script_fu_distress_selection(img, textLayer, 
	                                            127,     #Threshold 
												  8,     #Spread
												  4,     #Granularity  
												  2,     #Smooth
											   TRUE,	 #Smooth horizontally
											   TRUE)	 #Smooth _vertically
    
    pdb.gimp_context_set_pattern("Clipboard Image")  
    pdb.gimp_edit_bucket_fill(fabLayer,  
                   BUCKET_FILL_PATTERN,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y
    
    pdb.gimp_selection_none(img)
    patchLayer = pdb.gimp_image_flatten(img)
    pdb.gimp_item_set_name(patchLayer, "Patch")
    
    #Color option
    colorLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Layer", 100, LAYER_MODE_LCH_HUE)
    pdb.gimp_image_add_layer(img, colorLayer, 0)
    pdb.gimp_context_set_foreground(color)
    pdb.gimp_drawable_fill(colorLayer, FILL_FOREGROUND)
    patchLayer = pdb.gimp_image_merge_down(img, colorLayer, CLIP_TO_IMAGE)
      
    #Make active the bottom layer and done
    pdb.gimp_image_set_active_layer(img, patchLayer)   
   
   #Resets the previous user defaults
    pdb.gimp_context_set_defaults()

register(
    "pm_denim_patch_text_effect",
    "Creates a patch text on a denim",
    "Creates a patch text on a denim texture",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create Denim Patch Text effect",
    "",
    [
    (PF_TEXT, "customtext", "Text string", 'LOVE\nWINS'),
    (PF_FONT, "font", "Font face", "BubbleGum"),
    #(PF_SPINNER, "size", "Text size", 400, (400, 400, 1)),
    (PF_COLOR, "color", "Patch Color:", (255, 255, 255)),
    ],
    [],
    pm_denim_patch_text_effect, menu="<Image>/Pocholo-scripts/Create Denim Patch Text effect",
    domain=("gimp20", gimp.locale_directory))
main()	